% THIS FUNCTION COMPUTES THE HISTORICAL DECOMPOSITION FROM A SVAR WHERE
% BcompMC is the companion form matrix of the reduced form VAR without the mean and/or trend
% VC_epsMC is the covariance matrix of the reduced form (not in companion form - length(VC_epsMC) = nvars = Nbig
% Residmat is a (Tbig-vlag) x Nbig matrix of the reduced form residuls from the estimated VAR

function [path] = get_path(Bcomp, VC_eps, Y, cvec, Residmat);
%--------------------------------------------------------------------------
%
% DESCRIPTION:
%
% This routine computes counterfactual apths of variables in the VAR
% for a given sequence of reduced form VAR shocks
%
%--------------------------------------------------------------------------
%
% INPUTS:
%
% Bcomp:   	matrix with structure of estimated reduced-form coefficients in 
%           companion form excluding all the deterministic terms. 
%           This includes the estimated parameters in the partition 
%           n x (n x (n x vlag)). The remaining partition is made up of a 
%           diagonalic identity matrix that pins down the lead-lag relation 
%           between variables. The size is (n x vlag) x (n x vlag).  
%
% VC_eps:  	covariance matrix of reduced-form residuals, size n x n
% 
% Y:		Data, T x n
%
% cvec:		Constant term from VAR
%
% Residmat:	(T - vlag) x n matrix of reduced form residuals from the VAR
%
% Notes: 
% A. VC_eps denotes the variance-covariance decomposition of shocks of
% reduced-form model;
% B. Nbig denotes the number of variables;
% C. VC_epschol denotes the Choleski decomposition of the
% variance-covariance matrix of shocks of reduced-form model;
% D. stdepsvec denotes the standard deviations of shocks of reduced-form
% model.
%
%--------------------------------------------------------------------------
%
% OUTPUT:
%
% path:    Counterfactual path of endogenous variables T x n
%
%--------------------------------------------------------------------------
%
% Author:  Lee Smith June 7, 2013
%
%--------------------------------------------------------------------------
[Tbig,Nbig] = size(Y);
Nbigcomp 	= length(Bcomp);
vlag 		= Nbigcomp/Nbig;
path 		= zeros(Tbig,Nbig);
path_comp 	= zeros(Nbig*vlag,Tbig);
start_date  = vlag+1;
cvec_comp 	= [cvec; zeros(Nbig*(vlag-1),1)];
for ll = 1:1:vlag;
	upper = ll*Nbig;
	lower = upper - Nbig + 1;
	path_comp_start(lower:upper,1) = Y(start_date+1-ll,:)';	
end;	

path_comp(:,start_date) = path_comp_start;

%% NOW COMPUTE THE PATH OF VARIABLES (SIMULATE THE MODEL) WITH THE REDUCED FORM SHOCKS
for tt = start_date+1:1:Tbig;
	path_comp(:,tt) = cvec_comp + Bcomp*path_comp(:,tt-1) + [Residmat(tt-vlag,:)'; zeros(Nbig*(vlag-1),1)];
end;
path = path_comp(1:Nbig,:)';