% THIS FUNCTION COMPUTES THE HISTORICAL DECOMPOSITION FROM A SVAR WHERE
% BcompMC is the companion form matrix of the reduced form VAR without the mean and/or trend
% VC_epsMC is the covariance matrix of the reduced form (not in companion form - length(VC_epsMC) = nvars = Nbig
% Residmat is a (Tbig-vlag) x Nbig matrix of the reduced form residuls from the estimated VAR

function [Residmatcf] = match_m_path(Bcomp, VC_eps, Y, cvec, Residmat, m_path);
%--------------------------------------------------------------------------
%
% DESCRIPTION:
%
% This routine computes counterfactual series of VAR residuals by adjusting
% structural MP shocks to match a given path of money.
%
%--------------------------------------------------------------------------
%
% INPUTS:
%
% Bcomp:   	matrix with structure of estimated reduced-form coefficients in 
%           companion form excluding all the deterministic terms. 
%           This includes the estimated parameters in the partition 
%           n x (n x (n x vlag)). The remaining partition is made up of a 
%           diagonalic identity matrix that pins down the lead-lag relation 
%           between variables. The size is (n x vlag) x (n x vlag).  
%
% VC_eps:  	covariance matrix of reduced-form residuals, size n x n
% 
% Y:		Data, T x n
%
% cvec:		Constant term from VAR
%
% Residmat:	(T - vlag) x n matrix of reduced form residuals from the VAR
%
% Notes: 
% A. VC_eps denotes the variance-covariance decomposition of shocks of
% reduced-form model;
% B. Nbig denotes the number of variables;
% C. VC_epschol denotes the Choleski decomposition of the
% variance-covariance matrix of shocks of reduced-form model;
% D. stdepsvec denotes the standard deviations of shocks of reduced-form
% model.
%
%--------------------------------------------------------------------------
%
% OUTPUT:
%
% path:    Counterfactual path of endogenous variables T x n
%
%--------------------------------------------------------------------------
%
% Author:  Lee Smith June 7, 2013
%
%--------------------------------------------------------------------------
[Tbig,Nbig] = size(Y);
Nbigcomp 	= length(Bcomp);
vlag 		= Nbigcomp/Nbig;
path_comp 	= zeros(Nbig*vlag,Tbig);
path_comp 	= zeros(Nbig*vlag,Tbig);
start_date  = vlag+1;
cvec_comp 	= [cvec; zeros(Nbig*(vlag-1),1)];
for ll = 1:1:vlag;
	upper = ll*Nbig;
	lower = upper - Nbig + 1;
	path_comp_start(lower:upper,1) = Y(start_date+1-ll,:)';	
end;	

path_comp(:,start_date) = path_comp_start;
%%
%% MAP REDUCED FORM SHOCKS INTO STRUCTURAL SHOCKS
%% IF e_t (Nbig x 1) represents the reduced form VAR residual and eps_t denote the structural VAR residual  at time t,
%% then with the recursive identification scheme imposed by the Choleski decomposition, we have that at each time t
%% e_t = VC_epschol*eps_t. So we can recover eps_t from e_t using the equation eps_t = inv(VC_epschol)*e_t. 
VC_epschol = chol(VC_eps)';
inv_VC_epschol = inv(VC_epschol);
struct_shocks = Residmat*inv_VC_epschol';
struct_shockscf = struct_shocks;
Residmatcf = Residmat;
%%
for tt = start_date+1:1:Tbig;
	path_predicted = cvec_comp + Bcomp*path_comp(:,tt-1) + [Residmatcf(tt-vlag,:)'; zeros(Nbig*(vlag-1),1)];
	mp_shock = (m_path(tt) - path_predicted(4))/VC_epschol(4,4);
	%mp_shock = path_predicted(4)
	%m_path(tt) 
	struct_shockscf(tt-vlag,4) = struct_shocks(tt-vlag,4) + mp_shock;
	Residmatcf(tt-vlag,:) = struct_shockscf(tt-vlag,:)*VC_epschol';
	path_comp(:,tt) = cvec_comp + Bcomp*path_comp(:,tt-1) + [Residmatcf(tt-vlag,:)'; zeros(Nbig*(vlag-1),1)];
end;