%--------------------------------------------------------------------------
% BAYESIAN COMPUTATION OF POSTERIOR CONFIDENCE BANDS
% LEE SMITH
%--------------------------------------------------------------------------


function [VDpoint, VDup_2, VDup_1, VDlo_1, VDlo_2, VDmed] = confid_int_fevd(Bcomp,VC_eps,cvec,NMC,Y,CIperc,var_decomp_horizons);

%--------------------------------------------------------------------------
%
% DESCRIPTION:
%
% This function computes the impulse response functions and confidence intervals
%
%--------------------------------------------------------------------------
%
% INPUTS:
%
% Bcomp:   matrix with structure of estimated reduced-form coefficients in 
%           companion form excluding all the deterministic terms. 
%           This includes the estimated parameters in the partition 
%           n x (n x (n x vlag)). The remaining partition is made up of a 
%           diagonalic identity matrix that pins down the lead-lag relation 
%           between variables. The size is (n x vlag) x (n x vlag).     
%
% VC_eps:  covariance matrix of reduced-form residuals, size n x n.
%
% cvec:    matrix with estimated constants; this returns the scalar 0 
%           if no constant is included in the model; 
%           otherwise, this is a matrix of size n x 1.
%
% NMC:     number of Monte Carlo simulations for the density of simulated
%           posterior
%
% Y:       Raw Data
%
% CIperc:  this is a 2 x 1 or 4 x 1 vector with lower and upper percentiles for 
%           the confidence itervals that are computed and, eventually, plotted
%
% var_decomp_horizons: is a column vector defining which forecasts to 
%			compute var decomps for (i.e. var_decomp_horizons = [1; 12; 24; 60])
%
%--------------------------------------------------------------------------
%
% OUTPUT:
%
% VDpoint: point estimate of varince decomposition
%
% VDup:    upper bound of varince decomposition
%
% VDlo:    lower bound of varince decomposition
%
% VDmed:   median of varince decomposition distribution (at each horizon point)
%
%%----------------------------------------------------------------------------------------------------------------------------------
%% BACK OUT THE SIZE OF VARIOUS OBJECTS OF INTEREST
[Tbig,Nbig] = size(Y);
iscon  		= sum(abs(cvec))>0;
Nbig   		= length(VC_eps);
Nbigcomp 	= length(Bcomp);
vlag   		= length(Bcomp)/Nbig;
BIG_X 		= zeros(Tbig-vlag,Nbig*vlag+iscon);
if iscon == 1
	BIG_X(:,1) 	= ones(Tbig-vlag,1);
end;
for lags = 1:1:vlag;
	lower_index_row		= vlag+1-lags;
	upper_index_row		= Tbig-lags;
	upper_index_col		= Nbig*lags+iscon;
	lower_index_col		= upper_index_col - Nbig + 1;
	BIG_X(:,lower_index_col:upper_index_col) = Y(lower_index_row:upper_index_row,:); 	
end;

BIG_Y = Y(vlag+1:Tbig,:);

VDpoint = get_var_decomp(Bcomp, VC_eps, var_decomp_horizons);

lvd 		= length(var_decomp_horizons);
var_decomp 	= zeros(lvd,Nbig,Nbig);
VDlo_1 = zeros(size(VDpoint,1),size(VDpoint,2),size(VDpoint,3));
VDlo_2 = VDlo_1;
VDmed  = VDlo_1;
VDup_2 = VDlo_1;
VDup_1 = VDlo_1;

l_CI 		= length(CIperc);

%% SETUP FOR STANDARD BAYESIAN MCMC (UNINOFRMATIVE NORMAL/INVERSE-WISHART CONJUGATE PRIOR)
%%--------------------------------------------------------------
BIG_A 			= (BIG_X'*BIG_X)\BIG_X'*BIG_Y; 
vec_BIG_A 		= reshape(BIG_A,Nbig*(iscon+Nbig*vlag),1);
wishdof 		= Tbig;
% VC_eps is calcualted using MATLAB's 
% cov() function which divides the SSE by (Tbig-1).
% Therefore, 'scale=Tbig-1' undoes this so that we 
% are working with the raw SSE
scale			= Tbig-1;			
COVAR 			= inv(BIG_X'*BIG_X);

% NOTE COVAR, AS CALCULATED ABOVE, CAN BE TROUBLESOME FOR VARIABLES THAT ARE HIGHLY CORRELATED,
% IF YOU RUN INTO PROBLEMS TRY...
% lm = fitlm(BIG_X,BIG_Y(:,1),'Intercept',false);
% sigma2_reg = (y_reg-lm.Fitted)'*(y_reg-lm.Fitted)/(lm.NumObservations-lm.NumVariables);
% sigma2_ols = sigma2_reg;
% xtxinv     = lm.CoefficientCovariance/sigma2_ols;
% COVAR	   = xtxinv;

%% DO MC RUNS
for mm=1:NMC;
	draw = mm;
	workbar(mm/NMC,'Performing Monte Carlo Integration...','Progress');
	%% HERE I EMPLOY ANTITHETIC SAMPLING WHICH SPEEDS UP THE CONVERGENCE
	if mod(mm,2) == 1
		% odd draw again
		VC_eps_draw 		= iwishrnd(scale*VC_eps,wishdof);
		vec_BIG_A_COVAR		= kron(VC_eps_draw,COVAR);
		vec_A_draw_u		= mvnrnd(zeros(Nbig*(iscon+Nbig*vlag),1)',vec_BIG_A_COVAR);
		vec_BIG_A_draw		= vec_BIG_A + vec_A_draw_u'; 
	else
		% even draw - take reflection
		vec_BIG_A_draw		= vec_BIG_A - vec_A_draw_u'; 
	end;
	%%
	BIG_A_draw			= reshape(vec_BIG_A_draw,iscon+Nbig*vlag,Nbig);
	Bcomp_draw			=[BIG_A_draw(1+iscon:end,:)'; kron(eye(vlag-1), eye(Nbig)), ... 
									zeros((vlag-1)*Nbig, Nbig)];
	allvar_decomp(:,:,:,mm) = get_var_decomp(Bcomp_draw, VC_eps_draw, var_decomp_horizons);
end
%% GET LOWER, MEDIAN AND UPPER VARIABLES OF INTEREST FROM SIMULATED DISTRIBUTION
%% var_decomp

for ff=1:1:lvd;
    for ii=1:Nbig;
        for jj=1:Nbig;
            sliceloc   =  squeeze(allvar_decomp(ff,jj,ii,:));
			if l_CI == 2;
            VDpercsloc = quantile(sliceloc,[CIperc(1);0.5;CIperc(2)]);
            VDlo_2(ff,jj,ii)   = VDpercsloc(1);
			VDmed(ff,jj,ii)    = VDpercsloc(2);
            VDup_2(ff,jj,ii)   = VDpercsloc(3);
			elseif l_CI == 4;
			VDpercsloc = quantile(sliceloc,[CIperc(1);CIperc(2);0.5;CIperc(3);CIperc(4)]);
            VDlo_1(ff,jj,ii)   = VDpercsloc(1);
			VDlo_2(ff,jj,ii)   = VDpercsloc(2);
            VDmed(ff,jj,ii)    = VDpercsloc(3);
			VDup_2(ff,jj,ii)   = VDpercsloc(4);
			VDup_1(ff,jj,ii)   = VDpercsloc(5);
			end;
        end
    end
end