function [SRrule,LRrule,LRdiffrule] = VAR_mprule(Bcomp, VC_eps, MPIpos);


%--------------------------------------------------------------------------
%
% DESCRIPTION:
%
% This routine computes the SVAR-implied monetary policy rule
%
%--------------------------------------------------------------------------
%
% INPUTS:
%
% Bcomp:	matrix with structure of estimated reduced-form coefficients in 
%           companion form excluding all the deterministic terms. 
%           This includes the estimated parameters in the partition 
%           n x (n x (n x vlag)). The remaining partition is made up of a 
%           diagonalic identity matrix that pins down the lead-lag relation 
%           between variables. The size is (n x vlag) x (n x vlag).  
%
% VC_eps:  covariance matrix of reduced-form residuals, size n x n
%
%--------------------------------------------------------------------------
%
% OUTPUT:
%
% SRrule:		Short-run policy reaction rule (compares to Arais, Caldara, and Rubio-Ramirez)
%
% LRrule:		Long-run policy reactions (compares to Sims and Zha)
%
% LRdiffrule:	Long-run policy reactions (in difference form)
%
%--------------------------------------------------------------------------

Nbig       = length(VC_eps);
Nbigcomp   = length(Bcomp);
vlag	   = Nbigcomp/Nbig;
A0		   = inv(chol(VC_eps)');

%% PULL OUT IMPLIED MP EQUATION FROM ESTIMATED SVAR
MPlagcoefs = Bcomp(MPIpos,:)';
MPcurcoefs = A0(MPIpos,:)';

%% NORMALIZE COEFFICIENT ON MP INDICATOR TO EQUAL 1 
MPlagcoefs = MPlagcoefs/MPcurcoefs(MPIpos,1);
MPcurcoefs = MPcurcoefs/MPcurcoefs(MPIpos,1);


%% PULL OUT RESPONSE TO INDIVIDUAL VARIABLES
%% alpha=(rows,columns)=(variables,lags) = ([y,p,pcom,m]',[current,1st lag, 2nd lag, ...])
alpha = zeros(length(1:1:MPIpos),vlag+1);
alpha(1:MPIpos,1) = MPcurcoefs(1:MPIpos,1)';

for nn = 1:1:MPIpos
	for vv = 1:1:vlag
		alpha(nn,1+vv) = MPlagcoefs(nn+Nbig*(vv-1),1);
	end;
end;

delta = zeros(1,vlag+1);
delta = alpha(MPIpos,:);
alpha = alpha(1:MPIpos-1,:);

%% SHIFT CONTEMPORANEOUS REACTIONS TO RHS
%% AND SHIFT LAGS OF MP INDICATOR TO LHS
alpha(:,1)		= -1*alpha(:,1);
delta(1,2:end) 	= -1*delta(1,2:end);

SRrule = alpha(:,1);

%% CALCULATE SIMS-ZHA DENOMINATOR
sumdelta = sum(delta);

%% NOW CALCULATE LONG-RUN RESPONSE
for nn = 1:1:size(alpha,1);
	LRrule(nn,1) = sum(alpha(nn,:))/sumdelta;
end;


%% CALCULATE SIMS-ZHA DENOMINATOR FOR DIFF
cumsumdelta = sum(cumsum(delta));

%% NOW CALCULATE LONG-RUN RESPONSE FOR DIFF
for nn = 1:1:size(alpha,1);
		LRdiffrule(nn,1) = sum(cumsum(alpha(nn,:)))/cumsumdelta;
end;


SRrule(size(SRrule,1)+1,1) 		= SRrule(2,1)+SRrule(3,1);
LRrule(size(LRrule,1)+1,1) 		= LRrule(2,1)+LRrule(3,1);
LRdiffrule(size(LRdiffrule,1)+1,1) 	= LRdiffrule(2,1)+LRdiffrule(3,1);