%--------------------------------------------------------------------------
% BAYESIAN COMPUTATION OF POSTERIOR CONFIDENCE BANDS
% LEE SMITH
%--------------------------------------------------------------------------


function [MPrulepoint, MPruleup_2, MPruleup_1, MPrulelo_1, MPrulelo_2, MPrulemed] = confid_int_mprule(Bcomp,VC_eps,cvec,MPIpos,NMC,Y,CIperc);

%--------------------------------------------------------------------------
%
% DESCRIPTION:
%
% This function computes the confidence intervals for the SVAR-implied monetary policy rule
%
%--------------------------------------------------------------------------
%
% INPUTS:
%
% Bcomp:   	matrix with structure of estimated reduced-form coefficients in 
%           companion form excluding all the deterministic terms. 
%           This includes the estimated parameters in the partition 
%           n x (n x (n x vlag)). The remaining partition is made up of a 
%           diagonalic identity matrix that pins down the lead-lag relation 
%           between variables. The size is (n x vlag) x (n x vlag).     
%
% VC_eps:  	covariance matrix of reduced-form residuals, size n x n.
%
% MPIpos:	Position of monetary policy indicator in VAR
%
% NMC:     number of Monte Carlo simulations for the posterior density 
%
% Y:       Raw Data
%
% CIperc:  this is a 2 x 1 vector with lower and upper percentiles for 
%           the confidence itervals that are computed
%
%--------------------------------------------------------------------------
%
% OUTPUT:
%
% MPrulepoint: point estimate of policy reaction
%
% MPruleup:    upper bound of impulse responses 
%
% MPrulelo:    lower bound of impulse responses
%
% MPrulemed:   median of impulse response distribution (at each horizon point)
%
%%----------------------------------------------------------------------------------------------------------------------------------
%% BACK OUT THE SIZE OF VARIOUS OBJECTS OF INTEREST
[Tbig,Nbig] = size(Y);
iscon  		= sum(abs(cvec))>0;
Nbig   		= length(VC_eps);
Nbigcomp 	= length(Bcomp);
vlag   		= length(Bcomp)/Nbig;
BIG_X 		= zeros(Tbig-vlag,Nbig*vlag+iscon);
if iscon == 1
	BIG_X(:,1) 	= ones(Tbig-vlag,1);
end;
for lags = 1:1:vlag;
	lower_index_row		= vlag+1-lags;
	upper_index_row		= Tbig-lags;
	upper_index_col		= Nbig*lags+iscon;
	lower_index_col		= upper_index_col - Nbig + 1;
	BIG_X(:,lower_index_col:upper_index_col) = Y(lower_index_row:upper_index_row,:); 	
end;

BIG_Y = Y(vlag+1:Tbig,:);

[SRrulepoint,LRrulepoint,LRdiffrulepoint] = VAR_mprule(Bcomp, VC_eps, MPIpos);
MPrulepoint = [SRrulepoint,LRrulepoint,LRdiffrulepoint];

allmprules 	 = zeros(size(MPrulepoint,1),size(MPrulepoint,2),NMC);
MPruleup_1   = zeros(size(MPrulepoint,1), size(MPrulepoint,2));
MPrulelo_1   = MPruleup_1;
MPruleup_2	 = MPruleup_1;
MPrulelo_2	 = MPruleup_1;
MPrulemed  	 = MPruleup_1;

l_CI = length(CIperc);
%% SETUP FOR STANDARD BAYESIAN MCMC (UNINOFRMATIVE NORMAL/INVERSE-WISHART CONJUGATE PRIOR)
%%--------------------------------------------------------------
BIG_A 			= (BIG_X'*BIG_X)\BIG_X'*BIG_Y; 
vec_BIG_A 		= reshape(BIG_A,Nbig*(iscon+Nbig*vlag),1);
wishdof 		= Tbig;
% VC_eps is calcualted using MATLAB's 
% cov() function which divides the SSE by (Tbig-1).
% Therefore, 'scale=Tbig-1' undoes this so that we 
% are working with the raw SSE
scale			= Tbig-1;			
COVAR 			= inv(BIG_X'*BIG_X);

%% DO MC RUNS
for mm=1:NMC;
	draw = mm;
	workbar(mm/NMC,'Performing Monte Carlo Integration...','Progress');
	%% HERE I EMPLOY ANTITHETIC SAMPLING WHICH SPEEDS UP THE CONVERGENCE
	if mod(mm,2) == 1
		% odd draw again
		VC_eps_draw 		= iwishrnd(scale*VC_eps,wishdof);
		vec_BIG_A_COVAR		= kron(VC_eps_draw,COVAR);
		vec_A_draw_u		= mvnrnd(zeros(Nbig*(iscon+Nbig*vlag),1)',vec_BIG_A_COVAR);
		vec_BIG_A_draw		= vec_BIG_A + vec_A_draw_u'; 
	else
		% even draw - take reflection
		vec_BIG_A_draw		= vec_BIG_A - vec_A_draw_u'; 
	end;
	%%
	BIG_A_draw			= reshape(vec_BIG_A_draw,iscon+Nbig*vlag,Nbig);
	Bcomp_draw			=[BIG_A_draw(1+iscon:end,:)'; kron(eye(vlag-1), eye(Nbig)), ... 
									zeros((vlag-1)*Nbig, Nbig)];
	[SRruledraw,LRruledraw,LRdiffruledraw] = VAR_mprule(Bcomp_draw, VC_eps_draw, MPIpos);
	allmprules(:,:,mm) 	 = [SRruledraw,LRruledraw,LRdiffruledraw];   
end

%% GET LOWER, MEDIAN AND UPPER VARIABLES OF INTEREST FROM SIMULATED DISTRIBUTION
%% MPRULE CONFIDENCE INTERVALS
for ii=1:size(MPrulepoint,1);
	for jj=1:size(MPrulepoint,2);
		sliceloc   =  squeeze(allmprules(ii,jj,:));
		if l_CI == 2;
		MPrulepercsloc 		= quantile(sliceloc,[CIperc(1);0.5;CIperc(2)]);
		MPrulelo_2(ii,jj)   = MPrulepercsloc(1);
		MPrulemed(ii,jj)    = MPrulepercsloc(2);
		MPruleup_2(ii,jj)   = MPrulepercsloc(3);
		elseif l_CI == 4;
		MPrulepercsloc 		= quantile(sliceloc,[CIperc(1);CIperc(2);0.5;CIperc(3);CIperc(4)]);
		MPrulelo_1(ii,jj)   = MPrulepercsloc(1);
		MPrulelo_2(ii,jj)   = MPrulepercsloc(2);
		MPrulemed(ii,jj)    = MPrulepercsloc(3);
		MPruleup_2(ii,jj)   = MPrulepercsloc(4);
		MPruleup_1(ii,jj)   = MPrulepercsloc(5);
		end;
	end
end
